@@ -808,6 +808,10 @@ class TestXRImage(unittest.TestCase):
         img = xrimage.XRImage(data)
         with NamedTemporaryFile(suffix='.jpg') as tmp:
             img.save(tmp.name, fill_value=0)
+        # Jpeg fails without fill value (no alpha handling)
+        with NamedTemporaryFile(suffix='.jpg') as tmp:
+            # make sure fill_value is mentioned in the error message
+            self.assertRaisesRegex(OSError, "fill_value", img.save, tmp.name)
         # As PNG that support alpha channel
         img = xrimage.XRImage(data)
         with NamedTemporaryFile(suffix='.png') as tmp:
@@ -1022,6 +1026,17 @@ class TestXRImage(unittest.TestCase):
             np.testing.assert_allclose(file_data[1], exp[:, :, 1])
             np.testing.assert_allclose(file_data[2], exp[:, :, 2])
             np.testing.assert_allclose(file_data[3], 255)
+        # test .tiff too
+        with NamedTemporaryFile(suffix='.tiff') as tmp:
+            img.save(tmp.name)
+            with rio.open(tmp.name) as f:
+                file_data = f.read()
+            self.assertEqual(file_data.shape, (4, 5, 5))  # alpha band added
+            exp = np.arange(75).reshape(5, 5, 3)
+            np.testing.assert_allclose(file_data[0], exp[:, :, 0])
+            np.testing.assert_allclose(file_data[1], exp[:, :, 1])
+            np.testing.assert_allclose(file_data[2], exp[:, :, 2])
+            np.testing.assert_allclose(file_data[3], 255)
 
         data = xr.DataArray(da.from_array(np.arange(75).reshape(5, 5, 3), chunks=5),
                             dims=['y', 'x', 'bands'],
@@ -1212,6 +1227,30 @@ class TestXRImage(unittest.TestCase):
             with rio.open(tmp.name) as f:
                 self.assertEqual(len(f.overviews(1)), 2)
 
+        # auto-levels
+        data = np.zeros(25*25*3, dtype=np.uint8).reshape(25, 25, 3)
+        data = xr.DataArray(data, dims=[
+            'y', 'x', 'bands'], coords={'bands': ['R', 'G', 'B']})
+        img = xrimage.XRImage(data)
+        self.assertTrue(np.issubdtype(img.data.dtype, np.integer))
+        with NamedTemporaryFile(suffix='.tif') as tmp:
+            img.save(tmp.name, overviews=[], overviews_minsize=2)
+            with rio.open(tmp.name) as f:
+                self.assertEqual(len(f.overviews(1)), 4)
+
+        # auto-levels and resampling
+        data = np.zeros(25*25*3, dtype=np.uint8).reshape(25, 25, 3)
+        data = xr.DataArray(data, dims=[
+            'y', 'x', 'bands'], coords={'bands': ['R', 'G', 'B']})
+        img = xrimage.XRImage(data)
+        self.assertTrue(np.issubdtype(img.data.dtype, np.integer))
+        with NamedTemporaryFile(suffix='.tif') as tmp:
+            img.save(tmp.name, overviews=[], overviews_minsize=2,
+                     overviews_resampling='average')
+            with rio.open(tmp.name) as f:
+                # no way to check resampling method from the file
+                self.assertEqual(len(f.overviews(1)), 4)
+
     @unittest.skipIf(sys.platform.startswith('win'), "'NamedTemporaryFile' not supported on Windows")
     def test_save_tags(self):
         """Test saving geotiffs with tags."""
@@ -1956,21 +1995,42 @@ class TestXRImage(unittest.TestCase):
             return pil_obj
 
         img = xrimage.XRImage(data)
-        pi = mock.MagicMock()
-        img.pil_image = pi
-        res = img.apply_pil(dummy_fun, 'RGB')
-        # check that the pil image generation is delayed
-        pi.assert_not_called()
-        # make it happen
-        res.data.data.compute()
-        pi.return_value.convert.assert_called_with('RGB')
+        with mock.patch.object(xrimage, "PILImage") as pi:
+            pil_img = mock.MagicMock()
+            pi.fromarray = mock.Mock(wraps=lambda *args, **kwargs: pil_img)
+            res = img.apply_pil(dummy_fun, 'RGB')
+            # check that the pil image generation is delayed
+            pi.fromarray.assert_not_called()
+            # make it happen
+            res.data.data.compute()
+            pil_img.convert.assert_called_with('RGB')
 
         img = xrimage.XRImage(data)
-        pi = mock.MagicMock()
-        img.pil_image = pi
-        res = img.apply_pil(dummy_fun, 'RGB',
-                            fun_args=('Hey', 'Jude'),
-                            fun_kwargs={'chorus': "La lala lalalala"})
-        self.assertEqual(dummy_args, [({}, ), {}])
-        res.data.data.compute()
-        self.assertEqual(dummy_args, [(OrderedDict(), 'Hey', 'Jude'), {'chorus': "La lala lalalala"}])
+        with mock.patch.object(xrimage, "PILImage") as pi:
+            pil_img = mock.MagicMock()
+            pi.fromarray = mock.Mock(wraps=lambda *args, **kwargs: pil_img)
+            res = img.apply_pil(dummy_fun, 'RGB',
+                                fun_args=('Hey', 'Jude'),
+                                fun_kwargs={'chorus': "La lala lalalala"})
+            self.assertEqual(dummy_args, [({}, ), {}])
+            res.data.data.compute()
+            self.assertEqual(dummy_args, [(OrderedDict(), 'Hey', 'Jude'), {'chorus': "La lala lalalala"}])
+
+        # Test HACK for _burn_overlay
+        dummy_args = [(OrderedDict(), ), {}]
+
+        def _burn_overlay(pil_obj, *args, **kwargs):
+            dummy_args[0] = args
+            dummy_args[1] = kwargs
+            return pil_obj
+
+        img = xrimage.XRImage(data)
+        with mock.patch.object(xrimage, "PILImage") as pi:
+            pil_img = mock.MagicMock()
+            pi.fromarray = mock.Mock(wraps=lambda *args, **kwargs: pil_img)
+            res = img.apply_pil(_burn_overlay, 'RGB')
+            # check that the pil image generation is delayed
+            pi.fromarray.assert_not_called()
+            # make it happen
+            res.data.data.compute()
+            pil_img.convert.assert_called_with('RGB')
