@@ -47,6 +47,7 @@ from trollimage.image import check_image_format
 
 try:
     import rasterio
+    from rasterio.enums import Resampling
 except ImportError:
     rasterio = None
 
@@ -74,6 +75,16 @@ class RIOFile(object):
         self.rfile = None
         self.lock = threading.Lock()
 
+    @property
+    def width(self):
+        """Width of the band images."""
+        return self.kwargs['width']
+
+    @property
+    def height(self):
+        """Height of the band images."""
+        return self.kwargs['height']
+
     @property
     def closed(self):
         """Check if the file is closed."""
@@ -160,10 +171,15 @@ class RIOTag:
 class RIODataset:
     """A wrapper for a rasterio dataset."""
 
-    def __init__(self, rfile, overviews=None):
+    def __init__(self, rfile, overviews=None, overviews_resampling=None,
+                 overviews_minsize=256):
         """Init the rasterio dataset."""
         self.rfile = rfile
         self.overviews = overviews
+        if overviews_resampling is None:
+            overviews_resampling = 'nearest'
+        self.overviews_resampling = Resampling[overviews_resampling]
+        self.overviews_minsize = overviews_minsize
 
     def __setitem__(self, key, item):
         """Put the data chunk in the image."""
@@ -190,9 +206,19 @@ class RIODataset:
 
     def close(self):
         """Close the file."""
-        if self.overviews:
-            logger.debug('Building overviews %s', str(self.overviews))
-            self.rfile.build_overviews(self.overviews)
+        if self.overviews is not None:
+            overviews = self.overviews
+            # it's an empty list
+            if len(overviews) == 0:
+                from rasterio.rio.overview import get_maximum_overview_level
+                width = self.rfile.width
+                height = self.rfile.height
+                max_level = get_maximum_overview_level(
+                    width, height, self.overviews_minsize)
+                overviews = [2 ** j for j in range(1, max_level + 1)]
+            logger.debug('Building overviews %s with %s resampling',
+                         str(overviews), self.overviews_resampling.name)
+            self.rfile.build_overviews(overviews, resampling=self.overviews_resampling)
 
         return self.rfile.close()
 
@@ -263,6 +289,25 @@ def invert_scale_offset(scale, offset):
     return 1 / scale, -offset / scale
 
 
+@delayed(nout=1, pure=True)
+def delayed_pil_save(img, *args, **kwargs):
+    """Dask delayed saving of PIL Image object.
+
+    Special wrapper to handle `fill_value` try/except catch and provide a
+    more useful error message.
+
+    """
+    try:
+        img.save(*args, **kwargs)
+    except OSError as e:
+        # ex: cannot write mode LA as JPEG
+        if "A as JPEG" in str(e):
+            new_msg = ("Image mode not supported for this format. Specify "
+                       "`fill_value=0` to set invalid values to black.")
+            raise OSError(new_msg) from e
+        raise
+
+
 class XRImage(object):
     """Image class using an :class:`xarray.DataArray` as internal storage.
 
@@ -370,7 +415,7 @@ class XRImage(object):
         """
         kwformat = format_kwargs.pop('format', None)
         fformat = fformat or kwformat or os.path.splitext(filename)[1][1:]
-        if fformat in ('tif', 'jp2') and rasterio:
+        if fformat in ('tif', 'tiff', 'jp2') and rasterio:
             return self.rio_save(filename, fformat=fformat,
                                  fill_value=fill_value, compute=compute,
                                  keep_palette=keep_palette, cmap=cmap,
@@ -382,6 +427,7 @@ class XRImage(object):
     def rio_save(self, filename, fformat=None, fill_value=None,
                  dtype=np.uint8, compute=True, tags=None,
                  keep_palette=False, cmap=None, overviews=None,
+                 overviews_minsize=256, overviews_resampling=None,
                  include_scale_offset_tags=False,
                  **format_kwargs):
         """Save the image using rasterio.
@@ -405,6 +451,20 @@ class XRImage(object):
 
                     img.rio_save('myfile.tif', overviews=[2, 4, 8, 16])
 
+                If provided as an empty list, then levels will be
+                computed as powers of two until the last level has less
+                pixels than `overviews_minsize`.
+                Default is to not add overviews.
+            overviews_minsize (int): Minimum number of pixels for the smallest
+                overview size generated when `overviews` is auto-generated.
+                Defaults to 256.
+            overviews_resampling (str): Resampling method
+                to use when generating overviews. This must be the name of an
+                enum value from :class:`rasterio.enums.Resampling` and
+                only takes effect if the `overviews` keyword argument is
+                provided. Common values include `nearest` (default),
+                `bilinear`, `average`, and many others. See the rasterio
+                documentation for more information.
             include_scale_offset_tags (bool): Whether or not (default) to
                 include a ``scale`` and an ``offset`` tag in the data that would
                 help retrieving original data values from pixel values.
@@ -417,6 +477,7 @@ class XRImage(object):
         drivers = {'jpg': 'JPEG',
                    'png': 'PNG',
                    'tif': 'GTiff',
+                   'tiff': 'GTiff',
                    'jp2': 'JP2OpenJPEG'}
         driver = drivers.get(fformat, fformat)
 
@@ -515,7 +576,9 @@ class XRImage(object):
                 continue
 
         r_file.rfile.update_tags(**tags)
-        r_dataset = RIODataset(r_file, overviews)
+        r_dataset = RIODataset(r_file, overviews,
+                               overviews_resampling=overviews_resampling,
+                               overviews_minsize=overviews_minsize)
 
         to_store = (data.data, r_dataset)
         if da_tags:
@@ -551,7 +614,7 @@ class XRImage(object):
             format_kwargs['pnginfo'] = self._pngmeta()
 
         img = self.pil_image(fill_value, compute=False)
-        delay = img.save(filename, fformat, **format_kwargs)
+        delay = delayed_pil_save(img, filename, fformat, **format_kwargs)
         if compute:
             return delay.compute()
         return delay
@@ -571,19 +634,15 @@ class XRImage(object):
         return combine_scales_offsets(*scaling)
 
     @delayed(nout=1, pure=True)
-    def _delayed_apply_pil(self, fun, pil_args, pil_kwargs, fun_args, fun_kwargs,
+    def _delayed_apply_pil(self, fun, pil_image, fun_args, fun_kwargs,
                            image_metadata=None, output_mode=None):
-        if pil_args is None:
-            pil_args = tuple()
-        if pil_kwargs is None:
-            pil_kwargs = dict()
         if fun_args is None:
             fun_args = tuple()
         if fun_kwargs is None:
             fun_kwargs = dict()
         if image_metadata is None:
             image_metadata = dict()
-        new_img = fun(self.pil_image(*pil_args, **pil_kwargs), image_metadata, *fun_args, **fun_kwargs)
+        new_img = fun(pil_image, image_metadata, *fun_args, **fun_kwargs)
         if output_mode is not None:
             new_img = new_img.convert(output_mode)
         return np.array(new_img) / self.data.dtype.type(255.0)
@@ -599,8 +658,39 @@ class XRImage(object):
         The pil_args and pil_kwargs are passed to the `pil_image` method of the XRImage instance.
 
         """
-        new_array = self._delayed_apply_pil(fun, pil_args, pil_kwargs, fun_args, fun_kwargs,
-                                            self.data.attrs, output_mode)
+        if pil_args is None:
+            pil_args = tuple()
+        if pil_kwargs is None:
+            pil_kwargs = dict()
+        pil_image = self.pil_image(*pil_args, compute=False, **pil_kwargs)
+
+        # HACK: aggdraw.Font objects cause segmentation fault in dask tokenize
+        # Remove this when aggdraw is either updated to allow type(font_obj)
+        # or pycoast is updated to not accept Font objects
+        # See https://github.com/pytroll/pycoast/issues/43
+        # The last positional argument to the _burn_overlay function in Satpy
+        # is the 'overlay' dict. This could include aggdraw.Font objects so we
+        # completely remove it.
+        delayed_kwargs = {}
+        if fun.__name__ == "_burn_overlay":
+            from dask.base import tokenize
+            from dask.utils import funcname
+            func = self._delayed_apply_pil
+            if fun_args is None:
+                fun_args = tuple()
+            if fun_kwargs is None:
+                fun_kwargs = dict()
+            tokenize_args = (fun, pil_image, fun_args[:-1], fun_kwargs,
+                             self.data.attrs, output_mode)
+            dask_key_name = "%s-%s" % (
+                funcname(func),
+                tokenize(func.key, *tokenize_args, pure=True),
+            )
+            delayed_kwargs['dask_key_name'] = dask_key_name
+
+        new_array = self._delayed_apply_pil(fun, pil_image, fun_args, fun_kwargs,
+                                            self.data.attrs, output_mode,
+                                            **delayed_kwargs)
         bands = len(output_mode)
         arr = da.from_delayed(new_array, dtype=self.data.dtype,
                               shape=(self.data.sizes['y'], self.data.sizes['x'], bands))
